/***************************************************************************
                          controlador.cpp  -  description
                             -------------------
    begin                : Sun Feb 25 2001
    copyright            : (C) 2001 by Ricardo Markiewicz
    email                : gazerzoser@ciudad.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "controlador.h"

/* indices para los enemigos en el archivo del disco */
#define ENEMY_TORRETA 0
#define ENEMY_BASE 1
#define ENEMY_STORRETA 2

Controlador::Controlador(int tipo_juego, Pilotos *p1, Pilotos *p2)
{
	Uint32 i; /* lleva cuenta de numero de piloto */
	modo_juego = tipo_juego;
	cant_pilotos = 0;
	if (p1 != NULL)
		cant_pilotos++;
	if (p2 != NULL)
		cant_pilotos++;

	player[0] = p1;
	player[1] = p2;

	humano[0] = NULL;
	humano[1] = NULL;

	if (p2 == NULL)
		actual = new Niveles(player[0]->Level(), player[0]->Name(), NULL);	
	else
		actual = new Niveles(player[0]->Level(), player[0]->Name(), player[1]->Name());	
	
	for(i=0; i<cant_pilotos; i++) {
		if (player[i] != NULL) {
			puntos[i] = player[i]->Score();
			humano[i] = new NaveHumano(actual, this, player[i], i);
		}
	}

	/* inicio los parametros para las explociones */
	param_explocion.ef = PARTICULA_CIRCULAR;
	param_explocion.pos = Punto2D(0,0);
	param_explocion.dir = Punto2D(0,0);
	param_explocion.cant = 150;

	/* Creo paleta */
  /* create a suitable fire palette, this is crucial for a good effect */
  /* black to blue, blue to red, red to yellow, yellow to white*/
  
  for (i = 0; i < 32; ++i)
    {
      /* black to blue, 32 values*/
      param_explocion.colors[i] = Gra_GetColor(0, 0, i << 1);

      /* blue to red, 32 values*/
      param_explocion.colors[i + 32] = Gra_GetColor(i << 3, 0, 64 - (i << 1));

      /*red to yellow, 32 values*/
      param_explocion.colors[i + 64] = Gra_GetColor(210, i << 3, 0); //255

      /* yellow to white, 162 */
      param_explocion.colors[i + 96] = Gra_GetColor(210, 210, i<<2); //255
      param_explocion.colors[i + 128] = Gra_GetColor(210, 210, 0+(i<<2)); //255   64
      param_explocion.colors[i + 160] = Gra_GetColor(210, 210, 60+(i<<2)); //255 128
      param_explocion.colors[i + 192] = Gra_GetColor(210, 210, 130+i); //255  192
      param_explocion.colors[i + 224] = Gra_GetColor(210, 210, 180+i); //255  224
    } 

	/* color key */
	param_explocion.colors[255] = Gra_GetColor(0, 255, 0);

	/* bien, ya tengo creado el nivel y el humano, ahora cargo los datos de
	los enemigos del nivel */
	FILE *fp;
	int total_enemigos, cod, x, y, estado, pattern;
	char path_nivel[100];
	sprintf(path_nivel, "Mapas/Nivel%02i/enemigos.txt", player[0]->Level());
	fp = fopen(path_nivel, "r");
	if (fp != NULL) {
		fscanf(fp, "%i\n", &total_enemigos);
		for(i=0; i<total_enemigos; i++)
		{
			fscanf(fp, "%i %i %i %i %i\n", &cod, &x, &y, &estado, &pattern);
			switch (cod)
			{
				case (ENEMY_STORRETA):
					new STorreta(Punto2D(x,y), actual, this);
					break;
				case (ENEMY_TORRETA):
					new Torreta(Punto2D(x,y), actual, this);
					break;
				case (ENEMY_BASE):
					new Enemigos(Punto2D(x,y), actual, this, estado, pattern);
					break;
			}
		}
		fclose(fp);
	} else
		Gra_Debug("Controlador", "Constructor", "Imposible acceder al nivel", CRITICO);

	/* algunos chiches - cargo todo lo que se necesite */
	gameover_cod = Gra_LoadImage("Comunes/gameover.png", true);
	pausa_cod = Gra_LoadImage("Comunes/pausa.png", true);
	rocket_icon = Gra_LoadImage("Comunes/rocket_icon.png", true);
	sprintf(path_nivel, "Mapas/Nivel%02i/fin.png", player[0]->Level());
	fin_nivel = Gra_LoadImage(path_nivel, true);
	Gra_LoadImage("Comunes/item_back.png", true);
	Gra_LoadImage("Comunes/item_ion_up.png", true);
	Gra_LoadImage("Comunes/item_shield_up.png", true);
	Gra_LoadImage("Comunes/ion.bmp", false);
	Gra_LoadImage("Comunes/rocket.png", false);
	Gra_LoadImage("Comunes/faser.png", true);

	total_rockets[0] = total_rockets[1] = 2;
	rocket_launched[0] = rocket_launched[1] = 0;
	ion_items = 0;
	explode_sound = Gra_LoadWav("sound/efectos/explode.wav");
	item_sound = Gra_LoadWav("sound/efectos/getitem.wav");
	misiles_sound = Gra_LoadWav("sound/efectos/misiles.wav");
	sprintf(path_nivel, "sound/music/level%02i.mp3", player[0]->Level());
	background_music = Gra_LoadMp3(path_nivel);

	/* cargo teclas */
	for(i=0; i<cant_pilotos; i++) {
		Gra_getKeys(i, &key_player[i].tipo_mando, &key_player[i].up,&key_player[i].down,&key_player[i].left,&key_player[i].right,&key_player[i].fire1,&key_player[i].fire2);
	
		/* seteo el joystick si es necesario */
		if (key_player[i].tipo_mando >= 0) {
			if (SDL_NumJoysticks() > 0)
			{
				SDL_JoystickEventState(SDL_ENABLE);
				joystick[i] = SDL_JoystickOpen(key_player[i].tipo_mando);
				Gra_Debug("Controlador", "Constructor", "Iniciando control por Josytick", COMENTARIO);
			}
		} else {
			joystick[i] = NULL;
			key_player[i].tipo_mando = -1; /* pongo el teclado !! */
		}
	}
}

Controlador::~Controlador()
{
	char path[100];
	Gra_UnloadImage("Comunes/item_back.png");
	Gra_UnloadImage("Comunes/item_ion_up.png");
	Gra_UnloadImage("Comunes/item_shield_up.png");
	Gra_UnloadImage("Comunes/ion.bmp");
	Gra_UnloadImage("Comunes/rocket.png");
	Gra_UnloadImage("Comunes/pausa.png");
	Gra_UnloadImage("Comunes/gameover.png");
	Gra_UnloadImage("Comunes/faser.png");
	sprintf(path,"Mapas/Nivel%02i/fin.png", player[0]->Level());
	Gra_UnloadImage(path);
	Gra_UnloadMp3(background_music);

	for(int i=0; i<cant_pilotos; i++)
		if (joystick[i] != NULL)
			SDL_JoystickClose(joystick[i]);

	/* falta borrar las listas !!!! */
	Objetos *tmp;
	while ((tmp=los_malos.First()))
	{
		los_malos.Delete();
		delete tmp;
	}
	
	while ((tmp=los_buenos.First()))
	{
		los_buenos.Delete();
		delete tmp;
	}

	delete actual;
}

void Controlador::AddObjetoBueno(Objetos *add)
{
	if (add == NULL) return;
	los_buenos.Append(add);
}

void Controlador::AddObjetoMalo(Objetos *add)
{
	if (add == NULL) return;
	los_malos.Append(add);
}

void Controlador::DelObjetoBueno(Objetos *del)
{
	if (del == NULL) return;
	Objetos *tmp;
	for(tmp=los_buenos.First(); tmp; tmp=los_buenos.Next())
	{
		if (((tmp) == del) && (tmp!=humano[0]) && (tmp!=humano[1]))
		{
			los_buenos.Delete();
			delete tmp;
			break;
		}
	}
}

void Controlador::DelObjetoMalo(Objetos *del)
{
	if (del == NULL) return;
	Objetos *tmp;
	for(tmp=los_malos.First(); tmp; tmp=los_malos.Next())
	{
		if ((tmp) == del)
		{
			los_malos.Delete();
			delete tmp;
			break;
		}
	}
}

int Controlador::Play()
{
	CDConsole *cdplayer;
	Objetos *tmp_it;
	SDL_Event event;
	Uint32 init, fin;
	Uint32 frames=0;
	Uint8 *keystate;
	bool done = false, humano_fire[2], completo_nivel=false, show_fps=false;
	int i, snd_vol, snd_mus;

	humano_fire[0] = humano_fire[1] = 0;

	/* Verifico que el nivel ser cargo bien */
	if (!actual->isOk()) {
		/* Epa!!, algo salio mal .. aviso ... */
		NoLevel();
		return LOSE;
	}

	/* Continuo normalmente */
	Gra_getVolumen(&snd_vol, &snd_mus);
	if (snd_mus == -1) { /* usar CD Player !! */
		cdplayer = new CDConsole();
		cdplayer->Play(0);
	} else
		Gra_PlayMp3(background_music);

	actual->Dibujar();
	Gra_Print(ALIGN_H_CENTER, ALIGN_V_CENTER, FONT_COMIC, NULL, -1, false, actual->getName());
	Gra_Update();

	SDL_Delay(3*1000);

	init = SDL_GetTicks();
	Gra_InitFPS(60);
	while (!done)
	{
		/* verifico entrada usuario */
		while (SDL_PollEvent(&event))
		{
			switch (event.type)
			{
				case (SDL_KEYDOWN):
				{
					keystate = SDL_GetKeyState(NULL);
					for(i=0; i<cant_pilotos; i++) {
						if (key_player[i].tipo_mando == -1) {
							if ((keystate[key_player[i].left] == SDL_PRESSED) && (humano[i] != NULL))
									humano[i]->Alerones(-player[i]->moveX());
							if ((keystate[key_player[i].right] == SDL_PRESSED) && (humano[i] != NULL))
									humano[i]->Alerones(player[i]->moveX());
							if ((keystate[key_player[i].up] == SDL_PRESSED) && (humano[i] != NULL))
									humano[i]->Motor(player[i]->moveY());
							if ((keystate[key_player[i].down] == SDL_PRESSED) && (humano[i] != NULL))
									humano[i]->Motor(-player[i]->moveY());
							if (keystate[key_player[i].fire1] == SDL_PRESSED) {/* disparo arma seleccionada */
								humano_fire[i] = true;
								Gra_PlayWav(humano[i]->getPhaserSound());
							}
							if (keystate[key_player[i].fire2] == SDL_PRESSED)
								if ((rocket_launched[i] == 0) && (total_rockets[i] > 0))
									fireRocket(i);
						}
					} /* End for */
					if (keystate[SDLK_F3] == SDL_PRESSED)
						Gra_ToggleFullScreen();

					if (keystate[SDLK_ESCAPE] == SDL_PRESSED)
						done = true;
					if (keystate[SDLK_F12] == SDL_PRESSED)
						Gra_SaveScreen();
					if (keystate[SDLK_p] == SDL_PRESSED)
						Pausa();
					if (keystate[SDLK_F10] == SDL_PRESSED)
						show_fps = !show_fps;
					break;
				}
				case (SDL_KEYUP):
				{
					keystate = SDL_GetKeyState(NULL);
					for(i=0; i<cant_pilotos; i++) {
						if (key_player[i].tipo_mando == -1) {
							if (keystate[key_player[i].left] == SDL_RELEASED)
								if (humano[i]->Alerones() < 0)
									humano[i]->Alerones(0);
							if (keystate[key_player[i].right] == SDL_RELEASED)
								if (humano[i]->Alerones() > 0)
									humano[i]->Alerones(0);
							if (keystate[key_player[i].up] == SDL_RELEASED)
								if (humano[i]->Motor() > 0)
									humano[i]->Motor(0);
							if (keystate[key_player[i].down] == SDL_RELEASED)
								if (humano[i]->Motor() < 0)
									humano[i]->Motor(0);
							if (keystate[key_player[i].fire1] == SDL_RELEASED) /* disparo arma seleccionada */
								humano_fire[i] = false;
						}
					} /* End for */
					break;
				}
				case (SDL_JOYAXISMOTION):  /* Handle Joystick Motion */
				{
					for(i=0; i<cant_pilotos; i++) {
						if (event.jaxis.which == key_player[i].tipo_mando) {
							if ((event.jaxis.value < -6200) || (event.jaxis.value > 6200))
							{
								if( event.jaxis.axis == 0)
									humano[i]->Alerones((event.jaxis.value<0)?-HUMANO_VEL:HUMANO_VEL);
								if( event.jaxis.axis == 1)
									humano[i]->Motor((event.jaxis.value<0)?HUMANO_VEL:-HUMANO_VEL);
							}
							else
							{
								if( event.jaxis.axis == 0)
									humano[i]->Alerones(0);
								if( event.jaxis.axis == 1)
									humano[i]->Motor(0);
							}
						}
					} /* End For */
					break;
				}
				case (SDL_JOYBUTTONDOWN):
				{
					for(i=0; i<cant_pilotos; i++) {
						if (event.jbutton.which == key_player[i].tipo_mando) {
							if (event.jbutton.button == 0) {
								humano_fire[i] = true;
								Gra_PlayWav(humano[i]->getPhaserSound());
							}
							if (event.jbutton.button == 1)
								if ((rocket_launched[i] == 0) && (total_rockets[i] > 0))
									fireRocket(i);
						}
					}
					break;
				}
				case (SDL_JOYBUTTONUP):
				{
					for(i=0; i<cant_pilotos; i++) {
						if (event.jbutton.which == key_player[i].tipo_mando)
							if (event.jbutton.button == 0)
								humano_fire[i] = false;
					}
					break;
				}
			} /* End switch event.type */
		} /* End While */

		for(i=0; i<cant_pilotos; i++) {
			if (humano_fire[i]) {
				if (humano[i]->isAlive())
					humano[i]->Fire();
			}
		}

		if (cant_pilotos == 1) {
			if (!humano_fire[0])
				Gra_StopWav(humano[0]->getPhaserSound());
		} else {
			if ((!humano_fire[0]) && (!humano_fire[1]))
				Gra_StopWav(humano[0]->getPhaserSound());
		}

		/* muevo universo */
		actual->Mover();

		/* dibujo Universo */
		actual->Dibujar(); /* layers */
		checkItems(); /* verifico si agarre algo, para que se vea en pantalla antes de dibujar */

		/* Muevo y dibujo de una, para minimizar el tiempo cuando hay muchos elementos */
		for(tmp_it=los_malos.First(); tmp_it; tmp_it=los_malos.Next()) /* objetos Malos */
		{
			if (tmp_it->isAlive())
			{
				tmp_it->Mover();
				Punto2D tmp_pos = actual->ScreenPos(tmp_it->Pos());				
				if (((tmp_pos.Y()+tmp_it->RadioChoque()) > 0) && ((tmp_pos.Y()-tmp_it->RadioChoque()) < Gra_ScreenHeight()))
					tmp_it->Dibujar();
				else {
					if (tmp_pos.Y() > Gra_ScreenHeight()) {
						los_malos.Delete();
						delete tmp_it;
					}
				}
			} else {
				los_malos.Delete();
				delete tmp_it;
			}
		}

		for(tmp_it=los_buenos.First(); tmp_it; tmp_it=los_buenos.Next()) /* objetos Buenos */
		{
			if (tmp_it->isAlive())
			{
				tmp_it->Mover();
				Punto2D tmp_pos = actual->ScreenPos(tmp_it->Pos());
				if ((tmp_pos.Y()+tmp_it->RadioChoque()) > 0)
					tmp_it->Dibujar();
			} else {
				if ((tmp_it != humano[0]) && (tmp_it != humano[1])) {
					los_buenos.Delete();
					delete tmp_it;
				}
			}
		}

		/* Ya movi, ahora veo si chocan. Los cambio se ven reflejados en el proximo Update :-) */
		DetectarChoques();
		
		if (humano[1] == NULL) { /* Tengo un solo player */
			if (!humano[0]->isAlive())
				Gra_BlitImagen(gameover_cod, NULL, Punto2D(Gra_ScreenWidth()/2, Gra_ScreenHeight()/2));
			else {/* Verifico si se termino el nivel */
				if (!actual->isScrolling()) {
					if (los_malos.Count() == 0) {
						done = true;
						completo_nivel = true;
						ExitLevel();
					}
				}
			}
			
			/* algunos hacks para poner boludeces ...*/
			/* falta restringir la cantidad !!!, como? */
			/* FALTAN ITEM DE VIDA !! */
			if ((ion_items == 0) && ((humano[0]->getIon()/(float)MAX_ION) < 0.2))
			{
				if (rand()%500 == 1)
				{
					newItem( Punto2D(rand()%Gra_ScreenWidth(), -30), Items::ION_UP);
					ion_items++;
				}
			}
		} else {
			if ((!humano[0]->isAlive()) && (!humano[1]->isAlive()))
				Gra_BlitImagen(gameover_cod, NULL, Punto2D(Gra_ScreenWidth()/2, Gra_ScreenHeight()/2));
			else {
				if (!actual->isScrolling()) {
					if (los_malos.Count() == 0) {
						done = true;
						completo_nivel = true;
						ExitLevel();
					}
				}
			}
			/* algunos hacks para poner boludeces ...*/
			/* falta restringir la cantidad !!!, como? */
			if ((ion_items == 0) && (((humano[0]->getIon()/(float)MAX_ION) < 0.2) || ((humano[1]->getIon()/(float)MAX_ION) < 0.2)))
			{
				if (rand()%500 == 1)
				{
					newItem( Punto2D(rand()%(Gra_ScreenWidth()-10), -30), Items::ION_UP);
					ion_items++;
				}
			}
		}

		/* imprimo puntos, vidas, fps, etc */
		if (player[1] == NULL) {
			Gra_Print(580, 0, FONT_LCD, NULL, -1, false, "%06.00f", (float)puntos[0]);
			Gra_BlitImagen(rocket_icon, NULL, Punto2D(15, 470));
			Gra_Print(20, 460, FONT_LCD, NULL, -1, false, "%d", total_rockets[0]);
			if (show_fps)
				Gra_Print(40, 0, FONT_COMIC, NULL, -1, false, "fps: %.2f", Gra_FPS());
		} else {
			Gra_Print(200, 0, FONT_LCD, NULL, -1, false, "Jugador 1: %06.00f", (float)puntos[0]);
			Gra_Print(400, 0, FONT_LCD, NULL, -1, false, "Jugador 2: %06.00f", (float)puntos[1]);
			Gra_BlitImagen(rocket_icon, NULL, Punto2D(15, 470));
			Gra_BlitImagen(rocket_icon, NULL, Punto2D(620, 470));
			Gra_Print(20, 460, FONT_LCD, NULL, -1, false, "%d", total_rockets[0]);
			Gra_Print(600, 460, FONT_LCD, NULL, -1, false, "%d", total_rockets[1]);
			if (show_fps)
				Gra_Print(40, 0, FONT_COMIC, NULL, -1, false, "fps:%.2f", Gra_FPS());
		}

		Gra_SetSpeedFactor();
		Gra_Update();
		frames++;
	}
	fin = SDL_GetTicks();
	if (snd_mus == -1) { /* usar CD Player !! */
		cdplayer->Stop();
		delete cdplayer;
	} else
		Gra_StopMp3(background_music);

	printf("FPS : %f\n", (float)(frames)/((float)(fin-init)/(float)1000));
	switch (modo_juego) {
		case (CAMPAING_1):
			if (completo_nivel) {
				Gra_Debug("Controlador", "Play", "Has completado el nivel", COMENTARIO);
				player[0]->Score(puntos[0]);
				player[0]->incLevel();
				return WIN;
			}
			break;
		case (CAMPAING_2):
			if (completo_nivel) {
				player[0]->Score(puntos[0]);
				player[1]->Score(puntos[1]);
				player[0]->incLevel();
				player[1]->incLevel();
				return WIN;
			}
			break;
		case (SINGLE):
			return LOSE;
	}
	return LOSE;
}

void Controlador::DetectarChoques()
{
	Objetos *obj_1, *obj_2;
	int player;
	Punto2D dist;
	for(obj_1=los_malos.First(); obj_1; obj_1=los_malos.Next())
	{
		/* Si no esta en la pantalla no necesito verificar nada !! */
		if (!actual->isInside(obj_1)) continue;

		for(obj_2=los_buenos.First(); obj_2; obj_2=los_buenos.Next())
		{
			/* Si no esta en la pantalla no necesito verificar nada !! */
			if (!actual->isInside(obj_2)) continue;

			if (obj_1 == NULL) break;
			if (obj_2->Pos() == Punto2D(0,0)) continue;

			if ((obj_1->isAlive()) && (obj_2->isAlive()))
			{
				dist = obj_1->Pos();
				dist -= obj_2->Pos();
				/* primera aproximacion de detecion de choques */
				if (dist.Norma() <= (obj_1->RadioChoque()+obj_2->RadioChoque()))
				{
					obj_1->Daniar( obj_2->getDamage() );
					obj_2->Daniar( obj_1->getDamage() );
					player = obj_2->Player();
					if (!obj_2->isAlive())
					{
						if (obj_2->doExplosion())
						{
							param_explocion.pos = obj_2->Pos();
							new Particulas(actual, this, &param_explocion);
							Gra_PlayWav(explode_sound, 0);
						}
						if ((obj_2 != humano[0]) && (obj_2 != humano[1])) {
							los_buenos.Delete();
							delete obj_2;
							obj_2 = NULL;
						}
					}
					if (!obj_1->isAlive())
					{
						if (obj_1->doExplosion())
						{
							param_explocion.pos = obj_1->Pos();
							new Particulas(actual, this, &param_explocion);
							Gra_PlayWav(explode_sound, 0);
						}
						if (player != -1)
							puntos[player] += obj_1->getPuntos();
						los_malos.Delete();
						delete obj_1;
						obj_1 = NULL;
					}
				}
			}
		}
	}
}

void Controlador::checkItems()
{
	Items *tmp;
	Punto2D h_pos, dist;
	int i;
	for(tmp = items.First(); tmp; tmp=items.Next())
	{
		/* veo si el item "choca" con el humano */
		for(i=0; i<cant_pilotos; i++) {
			if (!humano[i]->isAlive()) continue;
			h_pos = actual->ScreenPos( humano[i]->Pos() );
			dist = tmp->Pos();
			dist -= h_pos;
			if (dist.Norma() <= (humano[i]->RadioChoque()+25))
			{
				switch (tmp->Type())
				{
					case (Items::LIFE_UP):
						humano[i]->Daniar(-100); /* hack para no hacer otra funcion */
					break;
					case (Items::ARMOR_UP):
					break;
					case (Items::ION_UP):
						ion_items--;
						humano[i]->setIon((int)(MAX_ION*(float)4/5));
					break;
				}
				Gra_PlayWav(item_sound, 0);
				items.Delete();
				delete tmp;
			}
			else
			{
				tmp->Mover();
				if (tmp->Pos().Y() > Gra_ScreenHeight())
				{
					if (tmp->Type() == Items::ION_UP) ion_items--;				
					items.Delete();
					delete tmp;
				}
				else
					tmp->Dibujar();
			}
		} /* End for i */
	} /* End for tmp */
}

void Controlador::deleteItem(Items *it)
{
	Items *tmp;
	for(tmp = items.First(); tmp; tmp=items.Next())
	{
		if (tmp == it)
		{
			if (tmp->Type() == Items::ION_UP) ion_items--;
			items.Delete();
			delete tmp;
		}
	}
}

void Controlador::newItem(Punto2D pos, int type)
{
	items.Append( new Items(pos, type) );
}

Punto2D Controlador::getMasCercano(Punto2D pos, bool buscar_malo)
{
	Objetos *tmp, *mas_cerca;
	float min_dist; /* distancia al objeto mas cerca */
	float dist;
	bool encontre=false;
	if (buscar_malo)
	{
		if (los_malos.Count() == 0) /* no hay objetos */
			return Punto2D(0,0);

		mas_cerca = NULL;
		min_dist = 99999; /* Muy lejos :-) */
		
		for(tmp=los_malos.First(); tmp; tmp=los_malos.Next())
		{
			if (!actual->isInside(tmp)) continue;

			dist = pos.dist(tmp->Pos());
			if ((dist < min_dist) && (tmp->doExplosion()))
			{
				mas_cerca = tmp;
				min_dist = dist;
				encontre = true;
			}
		}
		if (encontre)
			return mas_cerca->Pos();
		else
			return Punto2D(0,0);
	}
	else
	{
		if (los_buenos.Count() == 0) /* no hay objetos */
			return Punto2D(0,0);

		mas_cerca = los_buenos.First();
		min_dist = pos.dist(mas_cerca->Pos());

		for(tmp=los_buenos.Next(); tmp; tmp=los_buenos.Next())
		{
			dist = pos.dist(tmp->Pos());
			if ((dist < min_dist) && (tmp->doExplosion()))
			{
				mas_cerca = tmp;
				min_dist = dist;
			}
		}
		return mas_cerca->Pos();
	}
	return Punto2D(0,0);
}

Punto2D Controlador::getHumanoCerca(Punto2D pos)
{
	float min; /* distancia al mas cecar */
	
	/* el humano[0] siempre existe !! */
	if (humano[0]->isAlive())
		min = humano[0]->Pos().dist(pos);
	else
		min = 99999; /* en pantalla nunca pude ser !! */
	
	if (humano[1] != NULL)
		if (humano[1]->isAlive())
			if (humano[1]->Pos().dist(pos) < min)
				return humano[1]->Pos();
	
	return ((min==99999)?Punto2D(0,0):humano[0]->Pos());
}

void Controlador::fireRocket(int num_piloto)
{
	/* No hay nadie en pantalla aquien matar ... */
	if (getMasCercano(humano[num_piloto]->Pos(), true) == Punto2D(0,0))
		return;
	
	int y = -1*actual->Get_OffSide();
	new Rocket( Punto2D(30, y), actual, this, num_piloto);
	new Rocket( Punto2D(Gra_ScreenWidth()-30, y), actual, this, num_piloto);
	rocket_launched[num_piloto] = 2;
	total_rockets[num_piloto]--;
	Gra_PlayWav(misiles_sound, 0);
}

void Controlador::RocketDestroy(int num_piloto)
{
	rocket_launched[num_piloto]--;
}
 
Punto2D Controlador::humanoPos(int i)
{ 
	if (humano[i] == NULL) return Punto2D(0,0);

	return (humano[i]->isAlive())?humano[i]->Pos():Punto2D(0,0);
}

void Controlador::Pausa()
{
	int done=0;
	SDL_Event event;
	Uint8 *keystate;
	while (!done) {
		while (SDL_PollEvent(&event))
		{
			switch (event.type)
			{
				case (SDL_KEYDOWN):
				{
					keystate = SDL_GetKeyState(NULL);
					if (keystate[SDLK_p] == SDL_PRESSED)
						done = 1;
					if (keystate[SDLK_F12] == SDL_PRESSED)
						Gra_SaveScreen();
					break;
				}
			} /* End switch event.type */
		} /* End While */
		Gra_BlitImagen(pausa_cod, NULL, Punto2D(Gra_ScreenWidth()/2, Gra_ScreenHeight()/2));
		Gra_SetSpeedFactor();
		Gra_Update();
	}
}

void Controlador::NoLevel()
{
	int img = Gra_LoadImage("Comunes/no_level.png", true);
	int done=0;
	SDL_Event event;
	Uint8 *keystate;
	while (!done) {
		while (SDL_PollEvent(&event))
		{
			switch (event.type)
			{
				case (SDL_KEYDOWN):
				{
					keystate = SDL_GetKeyState(NULL);
					if (keystate[SDLK_ESCAPE] == SDL_PRESSED)
						done = 1;
					if (keystate[SDLK_F12] == SDL_PRESSED)
						Gra_SaveScreen();
					break;
				}
			} /* End switch event.type */
		} /* End While */
		Gra_Clean();
		Gra_BlitImagen(img, NULL, Punto2D(Gra_ScreenWidth()/2, Gra_ScreenHeight()/2));
		Gra_SetSpeedFactor();
		Gra_Update();
		/* restrinjo la velocidad del game ... */
	}
	Gra_UnloadImage("Comunes/no_level.png");
}

void Controlador::ExitLevel()
{
	Punto2D tmp;
	int i;
	tmp = actual->ScreenPos(humano[0]->Pos());
	/* Detengo la nave */
	for(i=0; i<cant_pilotos; i++) {
		humano[i]->Alerones(0);
		humano[i]->Motor(0);
	}
	/* Espero a salir */
	while (tmp.Y()+humano[0]->RadioChoque()>0) {
		actual->Dibujar();
		Gra_BlitImagen(fin_nivel, NULL, Punto2D(Gra_ScreenWidth()/2, Gra_ScreenHeight()/2));
		for(i=0; i<cant_pilotos; i++) {
			if (humano[i]->isAlive()) {
				humano[i]->Exit();
				humano[i]->Dibujar();
			}
		}
		Gra_Update();
		Gra_SetSpeedFactor();
		tmp = actual->ScreenPos(humano[0]->Pos());
	}
	SDL_Delay(1000);
}
