/***************************************************************************
                          controlador.h  -  description
                             -------------------
    begin                : Sun Feb 25 2001
    copyright            : (C) 2001 by Ricardo Markiewicz
    email                : gazerzoser@ciudad.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTROLADOR_H
#define CONTROLADOR_H

#include <SDL.h>
#include <stdarg.h>
#include "graficos_lib.h"
#include "matematica.h"
#include "niveles.h"
#include "rocket.h"
#include "objetos.h"
#include "navehumano.h"
#include "torreta.h"
#include "storreta.h"
#include "particulas.h"
#include "constantes.h"
#include "enemigos.h"
#include "items.h"
#include "lista.h"
#include "cdconsole.h"
#include "pilotos.h"

/**Esta clase se encarga de controlar y administrar
todo durante el juego, pero no se encarga
de los menues y esas yerbas. Controla la entrada
del usuario, las yerbas de redes (mas adelante...)
y los choques principalmente.
  *@author Ricardo Markiewicz
  */

class Controlador {
public: 
	enum {WIN, LOSE};
	Controlador(int tipo_juego, Pilotos *p1, Pilotos *p2=NULL);
	~Controlador();
	void AddObjetoBueno(Objetos *add);
	void AddObjetoMalo(Objetos *add);
	void DelObjetoBueno(Objetos *del);
	void DelObjetoMalo(Objetos *del);
	int Play();
	Punto2D humanoPos(int i);
	Punto2D getMasCercano(Punto2D pos, bool buscar_malo=false);
	Punto2D getHumanoCerca(Punto2D pos);
	void newItem(Punto2D pos, int type);
	void RocketDestroy(int num_piloto);
	inline int getExplodeSound() { return explode_sound; }
	void Pausa();
private:
	void NoLevel();
	void ExitLevel(); /* quiere decir que gane el nivel */

	/* va separada para que quede mas claro */
	void DetectarChoques();
	Niveles *actual;
	Lista<Objetos> los_malos;
	Lista<Objetos> los_buenos;
	
	/* Humano Stuff */
	int cant_pilotos;
	int total_rockets[2];
	int rocket_launched[2];
	t_Teclas key_player[2];
	NaveHumano *humano[2];
	Pilotos *player[2];

	SDL_Joystick *joystick[2];
	t_Part_Parametro param_explocion; /* parametros para explocion */

	/* imagenes */
	int gameover_cod, pausa_cod;
	int rocket_icon;
	int fin_nivel;
		
	int puntos[2];
	int ion_items;
	/* para los items */
	Lista<Items> items;
	void checkItems(); /* veo si el humano agarra algun Item o si tengo que borrar algun item */
	void deleteItem(Items *it);
	void fireRocket(int num_player);

	int explode_sound;
	int item_sound;
	int misiles_sound;
	int background_music;
	int modo_juego;
};

#endif
