/***************************************************************************
                          form.cpp  -  description
                             -------------------
    begin                : Sat Feb 24 2001
    copyright            : (C) 2001 by Ricardo Markiewicz
    email                : gazerzoser@ciudad.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "form.h"

Form::Form()
{
	funcion = NULL;
	action = Form::ERROR;
	focus = NULL;

	option_normal = option_select = -1;

	InitStyle();
}

Form::Form(void (*f)(void))
{
	funcion = f;
	focus = NULL;
	option_normal = option_select = -1;

	InitStyle();
}

Form::~Form()
{
	FormObj *tmp;
	for(tmp=lista.First(); tmp; tmp=lista.First()) {
		lista.Delete();
		delete tmp;
	}
	
	Gra_UnloadImage("Forms/option_select.png");
	Gra_UnloadImage("Forms/option_normal.png");

	Gra_UnloadImage("Forms/check_select.png");
	Gra_UnloadImage("Forms/check_normal.png");	
}

void Form::addObject(FormObj *o)
{
	lista.Append(o);
}

void Form::delObject(FormObj *mo)
{
	FormObj *o;
	for(o=lista.First(); (o != mo) && (o != NULL) ; o=lista.Next()) ;
	
	if (o != NULL)
		lista.Delete();
}

int Form::Go()
{
	SDL_Event event;
	done = false;
	FormObj *obj;
	if (!focus) {
		focus = lista.First();
		focus->setFocus();
	}
	Update();
	while(!done) {
		while (SDL_PollEvent(&event)) {
			switch (event.type) {
				case (SDL_KEYUP):
					if (event.key.keysym.sym == SDLK_F3) {
						Gra_ToggleFullScreen();
						break;
					}
					if (event.key.keysym.sym == SDLK_TAB) {
						NextFocus(focus);
						break;
					}
					if (event.key.keysym.sym == SDLK_ESCAPE) {
						Done();
						break;
					}
					if (event.key.keysym.sym == SDLK_F12)
						Gra_SaveScreen();
	
					focus->eventKeyUp(&event.key);
				break;
				case (SDL_KEYDOWN):
					if (event.key.keysym.sym == SDLK_RETURN) {
						action = OK;
						Done();
						break;
					}
					focus->eventKeyDown(&event.key);
				break;
				case (SDL_MOUSEBUTTONDOWN):
					obj = getObjIn(event.button.x, event.button.y);
					if (obj != NULL) {
						if (obj != focus) {
							focus->lostFocus();
							focus = obj;
							focus->setFocus();
						}
					}
					focus->eventMouseDown(&event.button);
				break;
				case (SDL_MOUSEBUTTONUP):
					obj = getObjIn(event.button.x, event.button.y);
					if (obj != NULL) {
						if (obj != focus) {
							focus->lostFocus();
							focus = obj;
							focus->setFocus();
						} else focus->eventMouseUp(&event.button);
					} else focus->eventMouseUp(&event.button);
				break;
				case (SDL_MOUSEMOTION):
					focus->eventMouseMove(&event.motion);
			}
		}
		Update();
	}
	return action;
}

void Form::NextFocus(FormObj *o)
{
	FormObj *tmp;

	for(tmp=lista.First(); tmp!=o; tmp=lista.Next()) ;
	
	while (1) { /* busco el siguiente que puede tener foco */
		tmp = lista.Next();
		if (tmp == NULL) {
			tmp = lista.First();
		}
		if (!tmp->canFocus()) continue;
		if (!tmp->Visible()) continue;
		break;
	}
	if (focus)
		focus->lostFocus();
	focus = tmp;
	focus->setFocus();
}

void Form::Done()
{
	done = true;
}

void Form::Update()
{
	FormObj *tmp;
	if (funcion != NULL)
		funcion();
	else
		Gra_Clean();

	for(tmp=lista.First(); tmp; tmp=lista.Next())
		if (tmp->Visible())
			tmp->Dibujar((tmp==focus)?true:false);
		
	Gra_Update();
}

FormObj *Form::getObjIn(int x, int y)
{
	FormObj *tmp;
	for(tmp=lista.First(); tmp; tmp=lista.Next()) {
		if ((tmp->isMouseOver(x, y)) && (tmp->Visible()) && (tmp->canFocus()))
			break;
	}
	return tmp;
}

void Form::setAction(int a)
{
	action = a;
}

void Form::InitStyle()
{
	color_fuente.r = 0;
	color_fuente.g = 0;
	color_fuente.b = 0;
	color_fuente.unused = 0;

	text_normal = color_fuente;
	
	text_focus.r = 255;
	text_focus.g = 255;
	text_focus.b = 255;
	text_focus.unused = 0;

	background = Gra_GetColor(200, 200, 200);
	list_item_select = Gra_GetColor(0, 0, 200);
	list_background = Gra_GetColor(255, 255, 255);
	text_background = Gra_GetColor(255, 255, 255);
	cursor_color = Gra_GetColor(0, 0, 0);
	
	option_select = Gra_LoadImage("Forms/option_select.png");
	option_normal = Gra_LoadImage("Forms/option_normal.png");

	check_select = Gra_LoadImage("Forms/check_select.png");
	check_normal = Gra_LoadImage("Forms/check_normal.png");
}
