/***************************************************************************
                          glist.cpp  -  description
                             -------------------
    begin                : Sun Jul 30 2000
    copyright            : (C) 2000 by Ricardo Markiewicz
    email                : gazer@linuxzone.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "glist.h"

GList::GList()
{
	first = current = last = NULL;
	numNodos = 0;
}

GList::~GList()
{
	LstNodo *n;
	while (first)
	{
		n = first;
		first=first->next;
		delete n;
	}
}

bool GList::Insert(LstNodo::Item it)
{
	LstNodo *newNodo = new LstNodo(it);
	newNodo->prev = NULL;
	newNodo->next = NULL;
	if (first == NULL)
		first = last = newNodo;
	else
	{
		LstNodo *tmp = new LstNodo(it);
		tmp->next = first;
		first->prev = tmp;
		first = tmp;
	}
	numNodos++;
	current = newNodo;
	return true;
}

bool GList::Insert(int pos, LstNodo::Item it)
{
	if (pos==0)
	{
		Insert(it);
		return true;
	}
	else if (pos==numNodos)
	{
		Append(it);
		return true;
	}
	LstNodo *nextNodo = Localizar(pos);
	if (nextNodo == NULL)
		return false;
	LstNodo *newNodo = new LstNodo(it);
	LstNodo *prevNodo = nextNodo->prev;
	prevNodo->next = newNodo;
	nextNodo->prev = newNodo;
	newNodo->prev = prevNodo;
	newNodo->next = nextNodo;
	current = newNodo;
	numNodos++;
	return true;
	
}

bool GList::Append(LstNodo::Item it)
{
	LstNodo *newNodo = new LstNodo(it);
	newNodo->prev = NULL;
	newNodo->next = NULL;
	if (first == NULL)
	{
		first = last = newNodo;
	}
	else
	{
		register LstNodo *tmp = first;
		for( ; tmp->next != NULL; tmp=tmp->next) {}
		newNodo->prev = tmp;
		tmp->next = newNodo;
	}
	numNodos++;
	last = newNodo;
	return true;
}

LstNodo *GList::Localizar(unsigned int pos)
{
	register LstNodo *tmp = first;
	for(unsigned int i=0 ; (i<pos) && tmp; i++, tmp=tmp->next) {}
	return tmp;
}

LstNodo::Item GList::Current()
{
	if (current)
		return current->data;
	return NULL;
}	

LstNodo::Item GList::Next()
{
	if (current)
		if (current->next)
		{
			current = current->next;
			return current->data;
		}
	return NULL;
}

LstNodo::Item GList::Prev()
{
	if (current)
		if (current->prev)
		{
			current = current->prev;
			return current->data;
		}
	return NULL;
}

LstNodo::Item GList::First()
{
	if (first)
	{
		current = first;
		return current->data;
	}
	return NULL;
}

LstNodo::Item GList::Last()
{
	if (last)
	{
		current = last;
		return current->data;
	}
	return NULL;
}

bool GList::Delete()
{
	if (!current) return false;
	LstNodo *prevNod, *nextNod;
	prevNod = current->prev;
	nextNod = current->next;
	delete current;
	if (prevNod)
	{
		prevNod->next = nextNod;
		current = prevNod;
	} else first = nextNod;
	if (nextNod)
	{
		nextNod->prev = prevNod;
		current = nextNod;
	} else last = prevNod;
	if ((!nextNod) && (!prevNod))
		current = first = last = NULL;
	numNodos--;
	return true;
}

bool GList::Delete(int i)
{
	current = Localizar(i);
	return Delete();
}

bool GList::DeleteFirst()
{
	current = first;
	return Delete();
}

bool GList::DeleteLast()
{
	current = last;
	return Delete();
}

LstNodo::Item GList::At(unsigned int pos)
{
	LstNodo *tmp = Localizar(pos);
	if (tmp)
		return tmp->data;
	return NULL;
}

int GList::Find(LstNodo::Item it)
{
	int pos;
	register LstNodo *tmp;
	for(pos=0, tmp=first; tmp && (tmp->data != it); tmp=tmp->next, pos++) {}
	if (tmp == NULL) return -1;
	return pos;
}
