/***************************************************************************
                          graficos_lib.cpp  -  description
                             -------------------
    begin                : Thu Feb 8 2001
    copyright            : (C) 2001 by Ricardo Markiewicz
    email                : gazerzoser@ciudad.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "graficos_lib.h"

typedef struct _l_imagenes {
	char *nombre; /* Guardo el nombre para comparar si piden 2 veces la misma imagen */
	int indice; /* indice de busqueda luego de ser cargada */
	SDL_Surface *pic; /* imagen en memoria */
	struct _l_imagenes *next;
} t_Item_Imagenes;

typedef enum {SND_WAV, SND_MP3} t_Snd;

typedef struct _l_sonidos {
	char *name;
	int indice;
	t_Snd tipo;
	int channel;
	Mix_Chunk *chunk;
	Mix_Music *music;
	struct _l_sonidos *next;
} t_Item_Sound;

SDL_Surface *screen;
bool lib_init=false; /* flag que indica si esta inicializada */
bool is_fullscreen;
bool have_sound;
t_Item_Imagenes *imagenes=NULL;
t_Item_Sound *sonidos=NULL;

int img_count=0;
int snd_count=0;

/* Fuentes */
TTF_Font *fonts[TOTAL_FONTS];
bool fonts_init=false;

/* Soporte Internacional */
struct {
	char **e;
	unsigned int total;
	char *nombre;
	char *file;
	bool init;
} Idioma;

/* Sonidos */
#define MAX_SONIDO 10
int volumen_musica;
int volumen_efectos;
t_Volumen volumen_enable;

t_Teclas Teclas[2];

/* Cursor del mouse */
struct {
	int mouse_pic;
	int x, y;
	SDL_Rect r;
	bool visible;
	SDL_Surface *back;
} Mouse;

struct _speed_ {
	float target_fps;
	float fps;
	float speedfactor;
	float max_factor, min_factor; /* para DEBUG de velocidad */
	Uint32 tickspersecond;
	Uint32 currentticks;
	Uint32 framedelay;
} fps_factor;

/* funciones privadas del front-end */
int imagenCargada(char *filename);
int sonidoCargado(char *filename);
SDL_Surface *getImagen(int cod);
t_Item_Sound *getSound(int cod);
Uint32 SumarColores(Uint32 col1, Uint32 col2);
Uint32 RestarColores(Uint32 col1, Uint32 col2);
/* alpha es aplicable a col1
   255 = opaco (retorna col1)
   0   = transparente (retorna col2)
*/
Uint32 SumarColoresA(Uint32 col1, Uint32 col2, Uint8 alpha);
bool EsIgualColor(Uint32 col1, Uint32 col2);
Uint32 DividirColor(Uint32 col, int factor);
Uint8 *GetSurfacePixel(SDL_Surface *s, int x, int y);
void SetSurfacePixel(SDL_Surface *s, int x, int y, Uint32 color);

int Gra_Init(int w, int h, int bpp, bool fullscreen)
{
	int audio_rate = 22050;
	Uint16 audio_format = AUDIO_S16; /* 16-bit stereo */
	int audio_channels = 2;
	int audio_buffers = 4096;

	/* inicializo SDL */
	if (lib_init != 0) return 0;
	srand(time(NULL));

	if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK|SDL_INIT_CDROM|SDL_INIT_AUDIO))
	{
		Gra_Debug("Engine Grafico", "Gra_Init", "SDL_GetError()", CRITICO);
		return -1;
	}

	/* Inicio SDL_mixer */
	if(Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers)) {
		Gra_Debug("Engine Sonido", "Gra_Init", Mix_GetError(), WARNING);
		have_sound = false;
	}
		have_sound = true;

	Uint32 flags;
	is_fullscreen = fullscreen;

#ifdef LINUX
	if (fullscreen)
		flags = SDL_FULLSCREEN | SDL_HWSURFACE;
	else
		flags = SDL_HWSURFACE | SDL_DOUBLEBUF;
	screen = SDL_SetVideoMode(w, h, bpp, flags);
#endif

#ifdef WIN32
	/* No uso SDL_HWSURFACE en pantalla completa porque se ve mal ? */
	if (fullscreen)
		flags = SDL_FULLSCREEN;
	else
		flags = SDL_HWSURFACE | SDL_DOUBLEBUF;
	screen = SDL_SetVideoMode(w, h, bpp, flags);
#endif

	if (screen == NULL)
	{
		Gra_Debug("Engine Grafico", "Gra_Init", SDL_GetError(), CRITICO);
		return -1;
	}
	
	/* repeticion de teclado */
	if (SDL_EnableKeyRepeat(1, SDL_DEFAULT_REPEAT_INTERVAL*3))
		Gra_Debug("Engine", "Gra_Init", SDL_GetError(), WARNING);

	lib_init = true;
	SDL_WM_SetCaption("Dexter Quad  -  " DEXTER_VERSION, "Dexter_Quad");

	Gra_Debug("Engine Grafico", "Gra_Init", "Inicio del Engine Completo", COMENTARIO);
	Gra_LoadFonts();
	
	SDL_ShowCursor( SDL_DISABLE );
	Mouse.mouse_pic = Gra_LoadImage("Comunes/mouse.png", true);
	Mouse.r.w = Gra_GetImagenWidth(Mouse.mouse_pic);
	Mouse.r.h = Gra_GetImagenHeight(Mouse.mouse_pic);
	Mouse.back = SDL_CreateRGBSurface(SDL_SWSURFACE, Gra_GetImagenWidth(Mouse.mouse_pic), \
			Gra_GetImagenHeight(Mouse.mouse_pic), 16, 0, 0, 0, 0);

	/* Incio FPS System */
	Gra_InitFPS(60);
	return 0;
}

SDL_Surface *Gra_ConvertirFormato(SDL_Surface *s)
{
	if (lib_init == false) {
		Gra_Debug("Engine Grafico", "Gra_ConvertirFormato", "No se ha inicializado Engine", CRITICO);
		return s;
	}

	SDL_Surface *tmp = SDL_DisplayFormat(s);

	if (tmp == NULL) {
		Gra_Debug("Engine Grafico", "Gra_ConvertirFormato", SDL_GetError(), CRITICO);
		return s;
	}
	SDL_FreeSurface(s);
	return tmp;
}

SDL_Surface *Gra_CopiarPantalla()
{
	return SDL_DisplayFormat(screen);
}

void Gra_FreeSurface(SDL_Surface *s)
{
	SDL_FreeSurface(s);
}

Uint32 Gra_GetColor(Uint8 r, Uint8 g, Uint8 b)
{
	if (lib_init)
		return SDL_MapRGB(screen->format, r, g, b);
	else
		return 0x00;
}

Uint32 Gra_GetColorA(Uint8 r, Uint8 g, Uint8 b, Uint8 a)
{
	if (lib_init)
		return SDL_MapRGBA(screen->format, r, g, b, a);
	else
		return 0x00;
}

void Gra_BlitImagen(SDL_Surface *img, SDL_Rect *from, Punto2D pos)
{
	/* primero convierto a SDL_Rect */
	SDL_Rect destino;
	destino.w = img->w;
	destino.h = img->h;
	if (from == NULL) {
		destino.x = (int)(pos.X()-img->w/2);
		destino.y = (int)(pos.Y()-img->h/2);
	} else {
		destino.x = (int)(pos.X()-from->w/2);
		destino.y = (int)(pos.Y()-from->h/2);
	}

	/* hago el Blit */
	SDL_BlitSurface(img, from, screen, &destino);
}

void Gra_BlitImagen(int cod, SDL_Rect *from, Punto2D pos)
{
	SDL_Surface *tmp = getImagen(cod);
	if (tmp == NULL)
		return;

	SDL_Rect destino;
	destino.w = tmp->w;
	destino.h = tmp->h;
	if (from == NULL) {
		destino.x = (int)(pos.X()-tmp->w/2);
		destino.y = (int)(pos.Y()-tmp->h/2);
	} else {
		destino.x = (int)(pos.X()-from->w/2);
		destino.y = (int)(pos.Y()-from->h/2);
	}

	SDL_BlitSurface(tmp, from, screen, &destino);
}

void Gra_BlitImagen(int cod, SDL_Rect *from, SDL_Rect *pos)
{
	SDL_Surface *tmp = getImagen(cod);
	if (tmp == NULL)
		return;
	SDL_BlitSurface(tmp, from, screen, pos);
}

void Gra_BlitImagen(SDL_Surface *img, SDL_Rect *from, SDL_Rect *pos)
{
	/* hago el Blit */
	SDL_BlitSurface(img, from, screen, pos);
}

int Gra_ToggleFullScreen()
{
	int a = SDL_WM_ToggleFullScreen(screen);
	if (a==0)
		Gra_Debug("Engine Grafico", "Gra_ToggleFullScreen", "No se puede cambiar de modo", WARNING);
	return a;
}

void Gra_Close()
{
	/* primero libero las imagenes cargadas ... */
	t_Item_Imagenes *tmp, *del;
	for(tmp=imagenes; tmp; ) {
		SDL_FreeSurface(tmp->pic);
		delete [] tmp->nombre;
		del = tmp;
		tmp = tmp->next;
		delete del;
	}

	/* libero los sonidos cargados */
	t_Item_Sound *tmp1, *del1;
	for(tmp1=sonidos; tmp1; ) {
		switch (tmp1->tipo) {
			case (SND_WAV):
				Mix_FreeChunk(tmp1->chunk);
			break;
			case (SND_MP3):
				Mix_FreeMusic(tmp1->music);
		}
		delete [] tmp1->name;
		del1 = tmp1;
		tmp1 = tmp1->next;
		delete del1;
	}

	Gra_Debug("Engine", "Performance", "Informe", COMENTARIO);
	printf("\tMaximo SpeedFactor : %.4f\n", fps_factor.max_factor);
	printf("\tMinimo SpeedFactor : %.4f\n", fps_factor.min_factor);

	SDL_ShowCursor( SDL_ENABLE );
	Gra_FreeFonts();
	Mix_CloseAudio();
	SDL_Quit();
}

void Gra_Clean()
{
	SDL_FillRect(screen ,NULL, 0x00);
}

void Gra_Update()
{
	if (Mouse.visible) {
		SDL_GetMouseState(&Mouse.x, &Mouse.y);
		Mouse.r.x = Mouse.x;
		Mouse.r.y = Mouse.y;
		SDL_BlitSurface(screen, &Mouse.r, Mouse.back, NULL);
		Gra_BlitImagen(Mouse.mouse_pic, NULL, &Mouse.r);
	}

	SDL_Flip(screen);
}

Uint16 Gra_ScreenWidth()
{
	return screen->w;
}

Uint16 Gra_ScreenHeight()
{
	return screen->h;
}

void Gra_SetCursorPos(int x, int y)
{
	Mouse.x = x;
	Mouse.y = y;
}

void Gra_GetCursorPos(int *x, int *y)
{
	*x = Mouse.x;
	*y = Mouse.y;
}

void Gra_HideCursor()
{
	Mouse.visible = false;
	Gra_BlitImagen(Mouse.back, NULL, &Mouse.r);
	Gra_Update();
}

void Gra_ShowCursor()
{
	Mouse.visible = true;
}

void Gra_SaveScreen()
{
	char n[14];
	sprintf(n, "shot_%05d.bmp", img_count++);
	SDL_SaveBMP(screen, n);
}

int Gra_LoadImage(char *filename, bool alpha)
{
	int cod = imagenCargada(filename);
	if (cod != -1)
		return cod;
	/* si llego a este punto no exite, y la cargo */
	SDL_Surface *pic = IMG_Load(filename);
	if (pic == NULL)
	{
		Gra_Debug("Engine Grafico", "Gra_LoadImagen FAIL", filename, CRITICO);
		return -1; /* no existe el BMP !! */
	}

	t_Item_Imagenes *tmp = new t_Item_Imagenes;
	tmp->next = NULL;
	tmp->indice = (++img_count);
	tmp->nombre = new char[strlen(filename)+1];
	strcpy(tmp->nombre, filename);

	if (alpha)
		tmp->pic = SDL_DisplayFormatAlpha(pic);
	else
		tmp->pic = SDL_DisplayFormat(pic);

	/* bien, ahora enlazo la lista */
	if (imagenes == NULL)
		imagenes = tmp;
	else
	{
		tmp->next = imagenes;
		imagenes = tmp;
	}
	SDL_FreeSurface(pic);
	Gra_Debug("Engine Grafico", "Gra_LoadImagen DONE", filename, COMENTARIO);
	return img_count;
}

void Gra_UnloadImage(char *filename)
{
	t_Item_Imagenes *tmp, *padre=NULL;
	for(tmp=imagenes;  tmp; )
	{
		if (strcmp(filename, tmp->nombre) == 0) break;
		padre = tmp;
		tmp = tmp->next;
	}
	if (tmp == NULL) return;

	if (padre == NULL) /* es el primero */
		imagenes = imagenes->next;
	else
		padre->next = tmp->next;

	Gra_Debug("Engine Grafico", "Gra_UnloadImage DONE", tmp->nombre, COMENTARIO);
	SDL_FreeSurface(tmp->pic);
	delete [] tmp->nombre;
	delete tmp;
}

void Gra_SetPixel(int x, int y, Uint32 color)
{
	if ((x>0) && (x<screen->w) && (y>0) && (y<screen->h))
	{
		Uint8 bpp = screen->format->BytesPerPixel;
		Uint8 *pixels = (Uint8 *)screen->pixels;
		pixels += (x*bpp + y*screen->pitch);
		*((Uint16 *)(pixels)) = (Uint16)color;
	}
}

void Gra_SetPixelA(int x, int y, Uint32 color, Uint8 alpha)
{
	if ((x>0) && (x<screen->w) && (y>0) && (y<screen->h))
	{
		Uint8 *pixels = Gra_GetPixel(x, y);
		*((Uint16 *)(pixels)) = (Uint16)SumarColoresA(color, *((Uint32 *)pixels), alpha);
	}
}

Uint8 *Gra_GetPixel(int x, int y)
{
	Uint8 bpp = screen->format->BytesPerPixel;
	Uint8 *pixels = (Uint8 *)screen->pixels;
	pixels += (x*bpp + y*screen->pitch);
	return pixels;
}

Uint32 Gra_GetPixelColor(int x, int y)
{
	Uint32 *pix = (Uint32 *)Gra_GetPixel(x,y);
	Uint32 col = Gra_GetColor(Gra_GetRed(*pix), Gra_GetGreen(*pix), Gra_GetBlue(*pix));
	return col;
}

void Gra_BlitRotateImage(int cod, float angulo, Punto2D pos, bool soft, bool alpha)
{
	SDL_Surface *rot;
	SDL_Surface *tmp = getImagen(cod);
	if (tmp == NULL) return;

	rot = Gra_RotateImagen(tmp, angulo, alpha);
	if (rot == NULL) { /* no requeria rotado o no se pudo */
		Gra_BlitImagen(tmp, NULL, pos);
		return;
	}

	if (soft)	Gra_BlurImagen(rot);

	Gra_BlitImagen(rot, NULL, pos);
	SDL_FreeSurface(rot);	
}

SDL_Surface *Gra_RotateImagen(SDL_Surface *s, float angle, bool alpha)
{
	int x,y,nx,ny;
	Uint8 r, g, b;
	float ca, sa;
	
	angle *= -1;
	ca = cos(angle);
	sa = sin(angle);
	Uint8 *out_pixels, *in_pixels;
	SDL_Surface *out;
	
	if (alpha == false)
		out = SDL_DisplayFormat(s);
	else
		out = SDL_DisplayFormatAlpha(s);
	
	SDL_LockSurface(out);
	SDL_LockSurface(s);

	for(y=0; y<s->h; y++)
		for(x=0; x<s->w; x++)
		{
			/* para rotar translado los puntos para que el centro
			de la imagen sea el (0,0) */
			nx = (int)((x-s->w/2) * ca - (y-s->h/2) * sa - 0.5 + s->w/2);
			ny = (int)((x-s->h/2) * sa + (y-s->h/2) * ca - 0.5 + s->h/2);
			if ((nx < 0) || (nx >= out->w) || (ny < 0) || (ny >= out->h))
				continue;
			out_pixels = (Uint8 *)out->pixels;
			out_pixels += (x*out->format->BytesPerPixel + y*out->pitch);
			in_pixels = (Uint8 *)s->pixels;
			in_pixels += (nx*s->format->BytesPerPixel + ny*s->pitch);
			//SDL_GetRGB(*((Uint32 *)in_pixels), s->format, &r, &g, &b);
			*((Uint16 *)out_pixels) = *((Uint16 *)in_pixels); //(Uint16)SDL_MapRGB(out->format, r, g, b);
		}
	SDL_UnlockSurface(s);
	SDL_UnlockSurface(out);
	return out;
}

void Gra_BlurImagen(SDL_Surface *s)
{
	int x, y, tx, ty;
	Uint32 r,g,b;
	SDL_Surface *out = SDL_DisplayFormat(s);
	SDL_LockSurface(s);
	SDL_LockSurface(out);
	for(y=1; y<s->h-1; y++)
	{
		for(x=1; x<s->w-1; x++)
		{
			Uint8 *dst;
			r = g = b = 0;
			for(ty=-1; ty<=1; ty++)
			{
				for(tx=-1; tx<=1; tx++)
				{
					Uint32 col;
					Uint8 *pix;

					pix = (Uint8 *)s->pixels;
					pix += (x+tx)*s->format->BytesPerPixel + (y+ty)*s->pitch;

					col = *((Uint32 *)pix);

					r += Gra_GetRed(col);
					g += Gra_GetGreen(col);
					b += Gra_GetBlue(col);
				}
			}
			r /= 9;
			g /= 9;
			b /= 9;
			dst = (Uint8 *)out->pixels;
			dst += x*out->format->BytesPerPixel + y*out->pitch;
			*((Uint32 *)dst) = SDL_MapRGB(screen->format, r, g, b);		
		}
	}

	SDL_UnlockSurface(s);
	SDL_UnlockSurface(out);
	SDL_BlitSurface(out, NULL, s, NULL);
	SDL_FreeSurface(out);
}

void Gra_Lock()
{
	SDL_LockSurface(screen);
}

void Gra_Unlock()
{
	SDL_UnlockSurface(screen);
}

void Gra_SetColorKey(int cod, Uint8 r, Uint8 g, Uint8 b)
{
	SDL_Surface *tmp;
	tmp = getImagen(cod);
	if (tmp==NULL) return;

	SDL_SetColorKey(tmp, SDL_SRCCOLORKEY, Gra_GetColor(r, g, b));
}

void Gra_SetAlpha(int i, Uint32 a, Uint8 alpha)
{
	if (i<0)
	{
		Gra_Debug("Engine Grafico", "Gra_SetAlpha", "No existe la imagen", WARNING);
		return;
	}
	if (alpha>255)
	{
		Gra_Debug("Engine Grafico", "Gra_SetAlpha", "el valor de Alpha no puede superar 255", WARNING);
		return;
	}
	SDL_Surface *tmp = getImagen(i);
	if (SDL_SetAlpha(tmp, a, alpha) != 0)
		Gra_Debug("Engine Grafico", "Gra_SetAlpha", SDL_GetError(), WARNING);
}

void Gra_DrawLine(int x1, int y1, int x2, int y2, Uint32 color)
{
	int x, y, dx, dy, c;
	float slope;

	dx = x2 - x1;  /* change in x */
	dy = y2 - y1;  /* change in y */

	slope = (float)dy / dx;

	Gra_Lock();
	if (abs(dx) >= abs(dy)) {
		/* draw left/right - right/left */
		c = (x1 <= x2) ? 1 : -1;
		for (x = x1; x != x2 + c; x += c) {
			y = (int)(y1 + ( (x - x1) * slope + 0.5 ));
			Gra_SetPixel(x, y, color);
		}
	}
	else {
		/* draw top/bottom - bottom/top */
		c = (y1 <= y2) ? 1 : -1;
		for (y = y1; y != y2 + c; y += c) {
			x = (int)(x1 + ( (y - y1) / slope + 0.5 ));
			Gra_SetPixel(x, y, color);
		}
	}
	Gra_Unlock();
}

Uint16 Gra_GetImagenWidth(int cod)
{
	SDL_Surface *tmp = getImagen(cod);
	if (tmp == NULL)
		return 0;
	return tmp->w;
}

Uint16 Gra_GetImagenHeight(int cod)
{
	SDL_Surface *tmp = getImagen(cod);
	if (tmp == NULL)
		return 0;
	return tmp->h;
}

Uint8 Gra_GetRed(Uint32 color)
{
	Uint16 temp;
	temp = color & screen->format->Rmask;
	temp >>= screen->format->Rshift;
	temp <<= screen->format->Rloss;
	return temp;
}

Uint8 Gra_GetGreen(Uint32 color)
{
	Uint16 temp;
	temp = color & screen->format->Gmask;
	temp >>= screen->format->Gshift;
	temp <<= screen->format->Gloss;
	return temp;
}

Uint8 Gra_GetBlue(Uint32 color)
{
	Uint16 temp;
	temp = color & screen->format->Bmask;
	temp >>= screen->format->Bshift;
	temp <<= screen->format->Bloss;
	return temp;
}

Uint8 Gra_GetAlpha(Uint32 color)
{
	Uint16 temp;
	temp = color & screen->format->Amask;
	temp >>= screen->format->Ashift;
	temp <<= screen->format->Aloss;
	return temp;
}

//
//
// Funciones internas para Graficos_Lib
//
//

Uint32 DividirColor(Uint32 col1, int factor)
{
	Uint8 r, g, b;
	SDL_GetRGB(col1, screen->format, &r, &g, &b);
	return Gra_GetColor(r/factor, g/factor, b/factor);
}


bool EsIgualColor(Uint32 col1, Uint32 col2)
{
	Uint8 r, g, b, r1, g1, b1;
	SDL_GetRGB(col1, screen->format, &r, &g, &b);
	SDL_GetRGB(col2, screen->format, &r1, &g1, &b1);
	return ((r == r1) && (g == g1) && (b == b1));
}

Uint32 RestarColores(Uint32 col1, Uint32 col2)
{
	Uint32 r, g, b;
	Uint32 temp;
	r = Gra_GetRed(col1);
	temp = Gra_GetRed(col2);
	if (r-temp<0)
		r = 0;
	else
		r -= temp;

	/* Get Green component */
	g = Gra_GetGreen(col1);
	temp = Gra_GetGreen(col2);
	if (g-temp<0)
		g = 0;
	else
		g -= temp;

	/* Get Blue component */
	b = Gra_GetBlue(col1);;
	temp = Gra_GetBlue(col2);
	if (b-temp<0)
		b = 0;
	else
		b -= temp;

	return Gra_GetColor(r,g,b);
}

Uint32 SumarColoresA(Uint32 col1, Uint32 col2, Uint8 alpha)
{
	if (alpha > 255)
	{
		Gra_Debug("Engine Grafico", "SumarColoresA", "Alpha fuera de rango", WARNING);
		return col1;
	}
	Uint8 r, g, b;
	r = Gra_GetRed(col1) * alpha / 255 + Gra_GetRed(col2) * (255-alpha)/255;
	g = Gra_GetGreen(col1) * alpha / 255 + Gra_GetGreen(col2) * (255-alpha)/255;
	b = Gra_GetBlue(col1) * alpha / 255 + Gra_GetBlue(col2) * (255-alpha)/255;
	
	return Gra_GetColor(r, g, b);
}

int imagenCargada(char *filename)
{
	t_Item_Imagenes *tmp;
	for(tmp = imagenes; tmp; tmp=tmp->next)
		if (strcmp(filename, tmp->nombre) == 0)
			return tmp->indice;
	return -1;
}

SDL_Surface *getImagen(int cod)
{
	t_Item_Imagenes *tmp;
	for(tmp = imagenes; tmp; tmp=tmp->next)
		if (tmp->indice == cod)
			return tmp->pic;
	return NULL;
}

Uint8 *GetSurfacePixel(SDL_Surface *s, int x, int y)
{
	Uint8 *pixel = (Uint8 *)s->pixels;
	pixel += (x*s->format->BytesPerPixel + y*s->pitch);
	return pixel;
}

void SetSurfacePixel(SDL_Surface *s, int x, int y, Uint32 color)
{
	if ((x<0) || (x>s->w) || (y<0) || (y>s->h))
		return;
	Uint8 *pixel = GetSurfacePixel(s,x,y);
	*((Uint16 *)pixel) = (Uint16)color;
}

Uint32 SumarColores(Uint32 col1, Uint32 col2)
{
	Uint32 r, g, b;
	Uint32 temp;
	r = Gra_GetRed(col1);
	temp = Gra_GetRed(col2);
	if (r+temp>255)
		r = 255;
	else
		r += temp;

	/* Get Green component */
	g = Gra_GetGreen(col1);
	temp = Gra_GetGreen(col2);
	if (g+temp>255)
		g = 255;
	else
		g += temp;

	/* Get Blue component */
	b = Gra_GetBlue(col1);
	temp = Gra_GetBlue(col2);
	if (b+temp>255)
		b = 255;
	else
		b += temp;

	return Gra_GetColor(r, g, b);
}


/* Fuentes !! */
void Gra_LoadFonts()
{
	if (TTF_Init() < 0) {
		Gra_Debug("Engine Fonts", "Gra_LoadFonts", TTF_GetError(), CRITICO);
		return;
	}
	
	/* Cargo las fuentes */
	fonts[0] = TTF_OpenFont("fonts/arial.ttf", 18);
	if (fonts[0] == NULL) Gra_Debug("Engine Fonts", "Gra_LoadFonts", "arial.ttf no existe", CRITICO);
	fonts[1] = TTF_OpenFont("fonts/comic.ttf", 20);
	if (fonts[1] == NULL) Gra_Debug("Engine Fonts", "Gra_LoadFonts", "comic.ttf no existe", CRITICO);
	fonts[2] = TTF_OpenFont("fonts/lcd.ttf", 18);
	if (fonts[2] == NULL) Gra_Debug("Engine Fonts", "Gra_LoadFonts", "lcd.ttf no existe", CRITICO);
	fonts[3] = TTF_OpenFont("fonts/credits.ttf", 28);
	if (fonts[3] == NULL) Gra_Debug("Engine Fonts", "Gra_LoadFonts", "credits.ttf no existe", CRITICO);
	fonts[4] = TTF_OpenFont("fonts/times.ttf", 16);
	if (fonts[4] == NULL) Gra_Debug("Engine Fonts", "Gra_LoadFonts", "times.ttf no existe", CRITICO);

	fonts_init = true;
}

void Gra_Print(int x, int y, int f, SDL_Color *color, int max_w, bool sombra, const char *fmt, ...)
{
	char		text[1024];
	va_list		ap;

	if (!fonts_init) return;
		
	if (fmt == NULL)
		return;

	va_start(ap, fmt);
	    vsprintf(text, fmt, ap);
	va_end(ap);

	if ((f < 0) || (f > TOTAL_FONTS)) return;
	if (fonts[f] == NULL) return;
	SDL_Rect dst, src;
	SDL_Surface *texto, *shadow, *tmp;
	static SDL_Color yellow = {0xdd, 0xdd, 0, 0};
	static SDL_Color gray = {0xdd, 0xdd, 0xdd, 0};
	int w, h;

	if (sombra) {
		shadow = TTF_RenderText_Solid(fonts[f], text, gray);
		tmp = SDL_DisplayFormat(shadow);
		SDL_FreeSurface(shadow);
		shadow = tmp;
	}

	if (color != NULL) {
		texto = TTF_RenderText_Solid(fonts[f], text, *color);
		tmp = SDL_DisplayFormat(texto);
		SDL_FreeSurface(texto);
		texto = tmp;
	}
	else {
		texto = TTF_RenderText_Solid(fonts[f], text, yellow);
		tmp = SDL_DisplayFormat(texto);
		SDL_FreeSurface(texto);
		texto = tmp;
	}
	if (texto != NULL) {
			if ((max_w < texto->w) && (max_w != -1))
				w = max_w;
			else
				w = texto->w;
			h = texto->h;

			if (x == ALIGN_H_CENTER)
				dst.x = (Uint16)(Gra_ScreenWidth()/2 - w/2 + 0.5);
			else
				dst.x = x;
			if (y == ALIGN_V_CENTER)
				dst.y = (Uint16)(Gra_ScreenHeight()/2 - h/2 + 0.5);
			else
				dst.y = y;
		
			dst.w = w;
			dst.h = h;
			src.x = src.y = 0;
			src.h = h;
			src.w = w;

			if (sombra) {
				dst.x += 2;
				dst.y += 2;
				SDL_BlitSurface(shadow, &src, screen, &dst);
				SDL_FreeSurface(shadow);
				dst.x -= 2;
				dst.y -= 2;
			}
			SDL_BlitSurface(texto, &src, screen, &dst);
			SDL_FreeSurface(texto);
	}
}

void Gra_FreeFonts()
{
	int i;
	for(i=0; i<TOTAL_FONTS; i++)
		if ((fonts[i] != NULL) && (fonts_init))
			TTF_CloseFont(fonts[i]);

	TTF_Quit();
}

void Gra_InitFPS(int _fps)
{
	fps_factor.target_fps = _fps;
	fps_factor.framedelay = SDL_GetTicks();
	fps_factor.tickspersecond = 1000;
	fps_factor.max_factor = 0;
	fps_factor.min_factor = 99;
}

void Gra_SetSpeedFactor()
{
	fps_factor.currentticks = SDL_GetTicks();
	fps_factor.speedfactor = (float)(fps_factor.currentticks - fps_factor.framedelay)/((float)fps_factor.tickspersecond/(float)fps_factor.target_fps);
	fps_factor.fps = fps_factor.target_fps / fps_factor.speedfactor;

	if (fps_factor.speedfactor > fps_factor.max_factor)
		fps_factor.max_factor = fps_factor.speedfactor;
	if (fps_factor.speedfactor < fps_factor.min_factor)
		fps_factor.min_factor = fps_factor.speedfactor;
	
	/* DEBUG */
	if (fps_factor.speedfactor > 5 )
		Gra_Debug("Engine Grafico", "SpeedFactor", "Me estoy llendo muy arriba en FPS (>5)", CRITICO);

	if (fps_factor.speedfactor > 2 )
		Gra_Debug("Engine Grafico", "SpeedFactor", "FPS disminuye un poco (>2)", WARNING);


	if (fps_factor.speedfactor < 1) {
		fps_factor.speedfactor = 1;
//		Gra_Debug("Engine Grafico", "SpeedFactor", "FPS es Optimo (<=1)", COMENTARIO);
	}
	fps_factor.framedelay = fps_factor.currentticks;
}

float Gra_SpeedFactor()
{
/* EN SUSPENSO!! ... algo no me gusta como anda :-(
	float tmp = floor(fps_factor.speedfactor);
	
	if ((fps_factor.speedfactor - tmp) > 0.5)
		return floor(fps_factor.speedfactor)+1;
*/	
	return floor(fps_factor.speedfactor);
}

float Gra_FPS()
{
		return fps_factor.fps;
}

void Gra_DrawBox(int x, int y, int w, int h, Uint32 fg, Uint32 bg, bool filled)
{
	int px, py;
	
	if (filled) {
		SDL_Rect r;
		r.x = x;
		r.y = y;
		r.w = w;
		r.h = h;
		SDL_FillRect(screen, &r, bg);
	}

	Gra_Lock();
	for(px=x, py=y; px<=x+w; px++)
		Gra_SetPixel(px, py, fg);

	for(px=x, py=y+h; px<=x+w; px++)
		Gra_SetPixel(px, py, fg);

	for(px=x, py=y; py<=y+h; py++)
		Gra_SetPixel(px, py, fg);

	for(px=x+w, py=y; py<=y+h; py++)
		Gra_SetPixel(px, py, fg);
	Gra_Unlock();
}

int Gra_TextWidth(int f, char *t)
{
	int w, h;
	TTF_SizeText(fonts[f], t, &w, &h);
	return w;
}

int Gra_TextHeight(int f, char *t)
{
	int w, h;
	TTF_SizeText(fonts[f], t, &w, &h);
	return h;
}

void Gra_DrawBox3D(int x, int y, int w, int h, Uint32 fg, Uint32 bg, bool filled, int type)
{
	Uint32 c1, c2, c3 ,c4;
	switch (type) {
		case (T3D_IN):
			c1 = Gra_GetColor(200, 200, 200);
			c2 = 0;
			c3 = Gra_GetColor(255, 255, 255);
			c4 = Gra_GetColor(128, 128, 128);
		break;
		case (T3D_OUT):
			c1 = Gra_GetColor(255, 255, 255);
			c2 = Gra_GetColor(200, 200, 200);
			c3 = 0;
			c4 = Gra_GetColor(128, 128, 128);
	}

	Gra_DrawBox(x, y, w, h, fg, bg, filled);
	/* Efecto 3D */
	Gra_DrawBox(x-1, y-1, w+2, 1, c1);
	Gra_DrawBox(x-1, y-1, 1, h+2, c1);
		
	Gra_DrawBox(x, y, w, 1, c2);
	Gra_DrawBox(x, y, 1, h, c2);

	Gra_DrawBox(x+w+1, y-1, 1, h+2, c3);
	Gra_DrawBox(x, y+h+1, w+2, 1, c3);

	Gra_DrawBox(x+w, y, 1, h, c4);
	Gra_DrawBox(x, y+h, w-1, 1, c4);
}

int sonidoCargado(char *filename)
{
	t_Item_Sound *tmp;
	for(tmp=sonidos; tmp; tmp=tmp->next)
	{
		if (strcmp(tmp->name, filename) == 0)
			return tmp->indice;
	}
	return -1;
}

t_Item_Sound *getSound(int cod)
{
	t_Item_Sound *tmp;
	for(tmp=sonidos; tmp; tmp=tmp->next)
	{
		if (tmp->indice == cod)
			return tmp;
	}
	return NULL;
}

int Gra_LoadWav(char *s)
{
	t_Item_Sound *nuevo;
	int ind = sonidoCargado(s);
	
	if (ind != -1) return ind;
	/* No existe .. Lo cargo */
	nuevo = new t_Item_Sound;
	nuevo->tipo = SND_WAV;
	nuevo->chunk = Mix_LoadWAV(s);
	if (nuevo->chunk == NULL) {
		Gra_Debug("Engine Sonido", "Gra_LoadWav FAIL", s, WARNING);
		Gra_Debug("Engine Sonido", "Gra_LoadWav FAIL", Mix_GetError(), WARNING);
		delete nuevo;
		return -1;
	}
	Gra_Debug("Engine Sonido", "Gra_LoadWav DONE", s, COMENTARIO);
	nuevo->channel = -1;
	nuevo->name = new char[strlen(s)+1];
	strcpy(nuevo->name, s);
	nuevo->indice = snd_count++;

	/* Hago la lista ... inserto al inicio */
	nuevo->next = sonidos;
	sonidos = nuevo;

	return nuevo->indice;
}

int Gra_LoadMp3(char *s)
{
	t_Item_Sound *nuevo;
	int ind = sonidoCargado(s);
	
	if (ind != -1) return ind;
	/* No existe .. Lo cargo */
	nuevo = new t_Item_Sound;
	nuevo->tipo = SND_MP3;
	nuevo->music = Mix_LoadMUS(s);
	if (nuevo->music == NULL) {
		Gra_Debug("Engine Sonido", "Gra_LoadMp3 FAIL", s, WARNING);
		Gra_Debug("Engine Sonido", "Gra_LoadMp3 FAIL", Mix_GetError(), WARNING);
		delete nuevo;
		return -1;
	}
	Gra_Debug("Engine Sonido", "Gra_LoadMp3 DONE", s, WARNING);
	nuevo->name = new char[strlen(s)+1];
	strcpy(nuevo->name, s);
	nuevo->indice = snd_count++;

	/* Hago la lista ... inserto al inicio */
	nuevo->next = sonidos;
	sonidos = nuevo;

	return nuevo->indice;
}

void Gra_UnloadWav(int cod)
{
	t_Item_Sound *tmp, *padre;
	tmp = getSound(cod);
	if (tmp==NULL) return;
		
	if (tmp == sonidos) {
		sonidos = sonidos->next;
	} else {
		for(padre=sonidos; padre; padre=padre->next)
			if (padre->next == tmp) break;
		padre->next = tmp->next;
	}
	
	/* borro tmp */
	Mix_FreeChunk(tmp->chunk);
	delete [] tmp->name;
	delete tmp;
}

void Gra_UnloadMp3(int cod)
{
	t_Item_Sound *tmp, *padre;
	tmp = getSound(cod);
	if (tmp==NULL) return;
		
	if (tmp == sonidos) {
		sonidos = sonidos->next;
	} else {
		for(padre=sonidos; padre; padre=padre->next)
			if (padre->next == tmp) break;
		padre->next = tmp->next;
	}
	
	/* borro tmp */
	Mix_FreeMusic(tmp->music);
	delete [] tmp->name;
	delete tmp;
}

void Gra_PlayWav(int cod, int times)
{
	if (cod<0) return;
	if (volumen_enable == VOLUMEN_OFF) return;

	t_Item_Sound *tmp = getSound(cod);
	if (tmp==NULL) return;
	
	if ((tmp->channel < 0) || (times != -1)) {
		Gra_VolumenWav(cod, volumen_efectos);
		tmp->channel = Mix_PlayChannel(-1, tmp->chunk, times);
	}
}

void Gra_PlayMp3(int cod)
{
	if (cod<0) return;
	if (volumen_enable == VOLUMEN_OFF) return;
		
	t_Item_Sound *tmp = getSound(cod);
	if (tmp==NULL) return;

	Gra_VolumenMp3(cod, volumen_musica);
	Mix_PlayMusic(tmp->music, 0);
}

void Gra_StopWav(int cod)
{
	if (cod < 0) return;
	if (volumen_enable == VOLUMEN_OFF) return;

	t_Item_Sound *tmp = getSound(cod);
	if (tmp==NULL) return;

	if (tmp->channel >= 0) {
		Mix_HaltChannel(tmp->channel);
		tmp->channel = -1;
	}
}

void Gra_StopMp3(int cod)
{
	if (cod<0) return;
	if (volumen_enable == VOLUMEN_OFF) return;
	
	Mix_HaltMusic();
}

void Gra_PauseMp3(int cod)
{
	if (volumen_enable == VOLUMEN_OFF) return;
	
	Mix_PauseMusic();
}

void Gra_VolumenMp3(int cod, int vol)
{
	if (cod<0) return;
	if (volumen_enable == VOLUMEN_OFF) return;
	if (vol < 0) return; /* esta usando CD !! */

	t_Item_Sound *tmp = getSound(cod);
	if (tmp==NULL) return;

	if (tmp->music != NULL)
		Mix_VolumeMusic(vol);
}

void Gra_VolumenWav(int cod, int vol)
{
	if (cod<0) return;
	if (volumen_enable == VOLUMEN_OFF) return;
		
	t_Item_Sound *tmp = getSound(cod);
	if (tmp==NULL) return;

	if (tmp->chunk != NULL)
		Mix_VolumeChunk(tmp->chunk, vol);
}

void Gra_setVolumen(int son, int mus)
{
	volumen_musica = mus;
	volumen_efectos = son;
}

void Gra_getVolumen(int *son, int *mus)
{
	*son = volumen_efectos;
	*mus = volumen_musica;
}

void Gra_Flush()
{
	FILE *fp;
	
	fp = fopen("dexter_cfg.dat", "w");
	fwrite(&volumen_musica, sizeof(int), 1, fp);
	fwrite(&volumen_efectos, sizeof(int), 1, fp);
	fwrite(&volumen_enable, sizeof(int), 1, fp);
	for(int i=0; i<2; i++) {
		fwrite(&Teclas[i].up, sizeof(int), 1, fp);
		fwrite(&Teclas[i].down, sizeof(int), 1, fp);
		fwrite(&Teclas[i].left, sizeof(int), 1, fp);
		fwrite(&Teclas[i].right, sizeof(int), 1, fp);
		fwrite(&Teclas[i].fire1, sizeof(int), 1, fp);
		fwrite(&Teclas[i].fire2, sizeof(int), 1, fp);
		fwrite(&Teclas[i].tipo_mando, sizeof(int), 1, fp);
	}
	fwrite(Gra_Idioma_File(), sizeof(char), strlen(Gra_Idioma_File()), fp);
	fclose(fp);
}

void Gra_setKeys(int i, int tipo, SDLKey up, SDLKey down, SDLKey left, SDLKey right, SDLKey fire1, SDLKey fire2)
{
	Teclas[i].up = up;
	Teclas[i].down = down;
	Teclas[i].left = left;
	Teclas[i].right = right;
	Teclas[i].fire1 = fire1;
	Teclas[i].fire2 = fire2;
	Teclas[i].tipo_mando = tipo;
}

void Gra_getKeys(int i, int *tipo, SDLKey *up, SDLKey *down, SDLKey *left, SDLKey *right, SDLKey *fire1, SDLKey *fire2)
{
	*up = Teclas[i].up;
	*down = Teclas[i].down;
	*left = Teclas[i].left;
	*right = Teclas[i].right;
	*fire1 = Teclas[i].fire1;
	*fire2 = Teclas[i].fire2;
	*tipo = Teclas[i].tipo_mando;
}

void Gra_setVolumenState(t_Volumen v)
{
	volumen_enable = v;
}

t_Volumen Gra_getVolumenState()
{
	return volumen_enable;
}

SDL_Surface *Gra_Screen()
{
	return screen;
}

void Gra_Debug(const char *modulo, const char *procedimiento, const char *mesage, t_Mensage tipo)
{
	switch (tipo) {
		case (CRITICO):
#ifdef LINUX
			cout << "[CRITICO] ";
#endif
#ifdef WIN32
			printf("[CRITICO] ");
#endif
		break;
		case (WARNING):
#ifdef LINUX
			cout << "[AVISO] ";
#endif
#ifdef WIN32
			printf("[AVISO] ");
#endif
		break;
		case (COMENTARIO):
#ifdef LINUX
			cout << "[COMENTARIO] ";
#endif
#ifdef WIN32
			printf("[COMENTARIO] ");
#endif
			break;
	}
#ifdef LINUX
	cout << modulo << ":" << procedimiento << " -> " << mesage << endl;
#endif
#ifdef WIN32
	printf("%s:%s -> %s\n", modulo, procedimiento, mesage);
#endif
}

void Gra_Idioma_Init(const char *f)
{
	FILE *fp = fopen(f, "r");
	if (fp != NULL) {
		int i;
		char tmp[255];
		
		Idioma.file = new char[strlen(f)+1];
		strcpy(Idioma.file, f);

		fscanf(fp, "%d\n", &Idioma.total);
		Idioma.e = new char *[Idioma.total];
		
		fscanf(fp, "%[^\n]", tmp);
		fgetc(fp); /* saco el \n */
		
		Idioma.nombre = new char[strlen(tmp)+1];
		strcpy(Idioma.nombre, tmp);

		for(i=0; i<Idioma.total; i++) {
			fscanf(fp, "%[^\n]", tmp);
			fgetc(fp); /* saco el \n */
			Idioma.e[i] = new char[strlen(tmp)+1];
			strcpy(Idioma.e[i], tmp);
		}
		Idioma.init = true;
		Gra_Debug("Engine Idioma", "Gra_Idioma_Init", "Carga completa", COMENTARIO);
	} else {
		Gra_Debug("Engine Idioma", "Gra_Idioma_Init", "No se encontro archivo de idioma", CRITICO);
		Idioma.e = NULL;
		Idioma.total = 0;
		Idioma.init = false;
	}
}

void Gra_Idioma_Free()
{
	if (!Idioma.init) return;

	for(int i=0; i<Idioma.total; i++)
		delete [] Idioma.e[i];
	delete [] Idioma.e;
	delete [] Idioma.nombre;
	delete [] Idioma.file;
	Idioma.total = 0;
}

char *Gra_Idioma_Get(unsigned int i)
{
	if (i>=Idioma.total) {
		Gra_Debug("Engine Idioma", "Gra_Idioma_Get", "I > Idioma.total", CRITICO);
		return NULL;
	}
	
	return Idioma.e[i];
}

char *Gra_Idioma_Name()
{
	return Idioma.nombre;
}

char *Gra_Idioma_File()
{
	return Idioma.file;
}
