/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : mi feb  7 01:50:28 ART 2001
    copyright            : (C) 2001 by Ricardo Markiewicz
    email                : gazerzoser@ciudad.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <SDL.h>
#include "menu.h"
#include "graficos_lib.h"
#include "controlador.h"
#include "scriptparser.h"
#include "startgame.h"
#include "pilotos.h"
#include "mpeg.h"

void SaveAndExit();
void LoadAndStart();
void UpdateList(FormObj *o);
void ListPilotoSelected(FormObj *o, FormObj *t);
void DrawDialog();

Lista<Pilotos> lst_pil;
SDL_Surface *back_dlg;

int main(int argc, char *argv[])
{
	Controlador *control;
	Pilotos *player[2]; /* usado en campaa */
	Pilotos *pilotos_tipo[6];

	int numero_piloto[2]; /* usado en multiplayer */

	player[0] = NULL;
	player[1] = NULL;
	numero_piloto[0] = numero_piloto[1] = -1;

	bool full=false, done=false;
	if (argc > 1)
	{
		for(int i=1; i < argc; i++)
			if (strcmp(argv[i], "-f") == 0)
				full = true;
			else
				Gra_Debug("Engine", "Parametro no reconocido", argv[i], WARNING);
	}
	if (Gra_Init(640, 480, 16, full))
		exit(1);

	LoadAndStart(); /* Debe ser llamada luego de Gra_Init */
	
	/* Son los pilotos por default */
	pilotos_tipo[FERNY] = new Pilotos(4, 6, 0, 0, FERNY, "Ferny");
	pilotos_tipo[JORGE] = new Pilotos(5, 5, 0, 0, JORGE, "Jorge");
	pilotos_tipo[RICKY] = new Pilotos(4, 4, 0, 0, RICKY, "Gazer");
	pilotos_tipo[JUANE] = new Pilotos(4, 4, 0, 0, JUANE, "Juane");
	pilotos_tipo[SAGAR] = new Pilotos(6, 6, 0, 0, SAGAR, "Sagar");
	pilotos_tipo[ZAFFO] = new Pilotos(4, 4, 0, 0, ZAFFO, "Zaffo");

	/* Muestro animacion de inicio */
	MPEGPlayer *intro;
	intro = new MPEGPlayer("video/intro.mpg", Gra_Screen());
	Gra_HideCursor();
	intro->Play();
	SDL_Delay(1000);
	delete intro;
	Gra_ShowCursor();

	int tipo_juego ;
	
	while (!done)
	{
		Menu *m = new Menu();
		switch (m->Choose())
		{
			case (Menu::PLAY):
				delete m;
				tipo_juego = SeleccionaJuego();
				switch (tipo_juego) {
					case (CAMPAING_1):
						player[0] = StartCampaing_1();
						player[1] = NULL;
					break;
					case (CAMPAING_2):
						StartCampaing_2(&numero_piloto[0], &numero_piloto[1]);
						if (numero_piloto[0] == -1) {
							player[0] = NULL;
							player[1] = NULL;
							break;
						}
						Gra_HideCursor();
						player[0] = pilotos_tipo[numero_piloto[0]];
						player[1] = pilotos_tipo[numero_piloto[1]];
						player[0]->Level(1);
						player[0]->Score(0);
						player[1]->Level(1);
						player[1]->Score(0);
						while (1) {
							control = new Controlador(tipo_juego, player[0], player[1]);
							if (control->Play() == Controlador::LOSE) break;
							delete control;
						}
						delete control;
						Gra_ShowCursor();
						player[0] = NULL;
						player[1] = NULL;
					break;
					case (SINGLE):
						int level;
						StartSingleLevel(&numero_piloto[0], &numero_piloto[1], &level);
						if (numero_piloto[0] != -1) {
							player[0] = pilotos_tipo[numero_piloto[0]];
							player[0]->Level(level);
							player[0]->Score(0);
						} else
							player[0] = NULL;
						if (numero_piloto[1] != -1) {
							player[1] = pilotos_tipo[numero_piloto[1]];
							player[1]->Level(level);
							player[1]->Score(0);
						}
						else
							player[1] = NULL;
						break;
					case (NONE):
						player[0] = player[1] = NULL;
				}
				if (player[0]) {
					Gra_HideCursor();
					control = new Controlador(tipo_juego, player[0], player[1]);
					control->Play();
					Gra_ShowCursor();
					delete control;
				}
				player[0] = NULL;
				player[1] = NULL;
				break;
			case (Menu::QUIT):
				done = true;
				delete m;
		}
	}
	/* Libero Idioma */
	Gra_Idioma_Free();

	Gra_Close();
	SaveAndExit();

	/* Libero los pilotos predeterminados */
	delete pilotos_tipo[FERNY];
	delete pilotos_tipo[JORGE];
	delete pilotos_tipo[RICKY];
	delete pilotos_tipo[JUANE];
	delete pilotos_tipo[SAGAR];
	delete pilotos_tipo[ZAFFO];

	Gra_Debug("Engine", "Principal", "Gracias por jugar Dexter Quad !!!", COMENTARIO);
	return EXIT_SUCCESS;
}

int ConfirmDialog(char *s, char *hok, char *hcancel)
{
	Form *form;
	Label *lbl;
	Button *ok, *cancel;
	int n, x, y, w, h;
	w = 340;
	h = 60;
	x = Gra_ScreenWidth()/2 - w/2;
	y = Gra_ScreenHeight()/2 - h/2;
	
	/* guardo la pantalla */
	Gra_HideCursor();
	back_dlg = Gra_CopiarPantalla();
	Gra_ShowCursor();

	form = new Form(&DrawDialog);
	
	lbl = new Label(form, x+5, y, FONT_TIMES, s);
	
	if (hok)
		ok = new Button(form, x+5, y+h-30, FONT_TIMES, hok, Form::OK);
	if (hcancel)
		cancel = new Button(form, x+w-10-Gra_TextWidth(FONT_TIMES, hcancel), y+h-30, FONT_TIMES, hcancel, Form::CANCEL);
	
	n = form->Go();
	
	delete form;

	Gra_FreeSurface(back_dlg);
	return n;
}

void DrawDialog()
{
	int x, y, w, h;
	w = 340;
	h = 60;
	x = Gra_ScreenWidth()/2 - w/2;
	y = Gra_ScreenHeight()/2 - h/2;
	Gra_BlitImagen(back_dlg, NULL, NULL);
	Gra_DrawBox3D(x, y, w, h, 0, Gra_GetColor(0xdd, 0xdd, 0xdd), true, T3D_OUT);
}

void SaveAndExit()
{
	int n;
	Pilotos *tmp;
	FILE *fp = fopen("dexter.dat", "w");
	
	n = lst_pil.Count();
	fwrite((void *)(&n), sizeof(int), 1, fp);

	for(tmp=lst_pil.First(); tmp; tmp=lst_pil.First()) {
		tmp->Save(fp);
		lst_pil.Delete();
		delete tmp;
	}
	
	fclose(fp);
}

void LoadAndStart()
{
	FILE *fp;
	int n;
	char tmp[255];
	fp = fopen("dexter.dat", "r");
	
	if (fp != NULL) {
		fread(&n, sizeof(int), 1, fp);
		for( ; n>0; n--)
			lst_pil.Append( new Pilotos(fp) );
		
		fclose(fp);
	}

	/* Cargo los parmetros de sonido y configuracin de teclado */
	int mus, son;
	t_Volumen estado;
	fp = fopen("dexter_cfg.dat", "r");
	SDLKey up[2], down[2], left[2], right[2], fire1[2], fire2[2];
	int tipo[2];

	if (fp != NULL) {
		fread(&mus, sizeof(int), 1, fp);
		fread(&son, sizeof(int), 1, fp);
		fread(&estado, sizeof(int), 1, fp);
		for(int i=0; i<2; i++) {
			fread(&up[i], sizeof(int), 1, fp);
			fread(&down[i], sizeof(int), 1, fp);
			fread(&left[i], sizeof(int), 1, fp);
			fread(&right[i], sizeof(int), 1, fp);
			fread(&fire1[i], sizeof(int), 1, fp);
			fread(&fire2[i], sizeof(int), 1, fp);
			fread(&tipo[i], sizeof(int), 1, fp);
		}
		int cc=0;
		char c;
		while ((c=fgetc(fp)) != EOF)
			tmp[cc++] = c;
		tmp[cc] = '\0';
		if (cc == 0) /* El archivo no contenia Idioma todavia!! */
			sprintf(tmp, "Idioma/es.idm");
		fclose(fp);
	} else {
		mus = son = MIX_MAX_VOLUME/2;
		estado = VOLUMEN_ON;
		up[0] = SDLK_UP;
		down[0] = SDLK_DOWN;
		left[0] = SDLK_LEFT;
		right[0] = SDLK_RIGHT;
		fire1[0] = SDLK_o;
		fire2[0] = SDLK_l;
		up[1] = SDLK_t;
		down[1] = SDLK_g;
		left[1] = SDLK_f;
		right[1] = SDLK_h;
		fire1[1] = SDLK_q;
		fire2[1] = SDLK_a;
		tipo[0] = tipo[1] = -1;
		sprintf(tmp, "Idioma/es.idm");
	}

	Gra_setVolumen(son, mus);
	Gra_setVolumenState(estado);
	Gra_setKeys(0, tipo[0], up[0], down[0], left[0], right[0], fire1[0], fire2[0]);
	Gra_setKeys(1, tipo[1], up[1], down[1], left[1], right[1], fire1[1], fire2[1]);
	Gra_Idioma_Init(tmp);
}

int GetPilotIndex(int i)
{
	switch (i) {
		case (FERNY):
			return Gra_LoadImage("pilotos/ferny.png");
		case (JORGE):
			return Gra_LoadImage("pilotos/jorge.png");
		case (RICKY):
			return Gra_LoadImage("pilotos/ricky.png");
		case (JUANE):
			return Gra_LoadImage("pilotos/juane.png");
		case (SAGAR):
			return Gra_LoadImage("pilotos/sagar.png");
		case (ZAFFO):
			return Gra_LoadImage("pilotos/zaffo.png");
	}
	return -1;
}
