/***************************************************************************
                          textfield.cpp  -  description
                             -------------------
    begin                : Sat Feb 24 2001
    copyright            : (C) 2001 by Ricardo Markiewicz
    email                : gazerzoser@ciudad.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "textfield.h"

TextField::TextField(Form *p, int x, int y, int w, int h, int max, int font):FormObj(p)
{
	r.x = x;
	r.y = y;
	r.w = w;
	r.h = h;
	total = max;
	offset = cant = 0;
	fuente = font;
	texto = new char[total+1];
	for(int i=0; i<total; i++)
		texto[i] = ' ';
	texto[0] = '\0';
	pos = 0;
	shift = false;
	insert = true;
}

TextField::~TextField()
{
}

void TextField::Dibujar(bool focus)
{
	int w, h;
	w = Gra_TextWidth(fuente, texto+offset);
	h = Gra_TextHeight(fuente, texto+offset);
	if ((w) > (r.w-2)) {
		offset++;
		Dibujar(focus);
		return;
	}

	Gra_DrawBox3D(r.x, r.y, r.w, r.h, 0, parent->text_background, true);
	
	if (w>0)
		Gra_Print(r.x+3, r.y+(r.h-h)/2, fuente, &parent->text_normal, r.w-3, false, texto+offset);
	if (focus)
		Gra_DrawBox(r.x+w+1, r.y+(r.h-h+4)/2, 1, h-6, parent->cursor_color, 0, false);
}

void TextField::setFocus()
{
}

void TextField::setCursorPos(int _p)
{
	if (_p < 0)
		_p = 0;
	
	if (_p > cant)
		_p = cant;
	
	if (_p > total)
		_p = total;
	
	pos = _p;
	
	if (offset > pos)
		offset = pos;
}

void TextField::eventKeyDown(SDL_KeyboardEvent *key)
{
	switch (key->keysym.sym) {
		case (SDLK_RSHIFT):
		case (SDLK_LSHIFT):
			shift = true;
		break;
	}
}

void TextField::eventKeyUp(SDL_KeyboardEvent *key)
{
	switch (key->keysym.sym) {
		case (SDLK_BACKSPACE):
			if (pos != 0) {
				for(int i = pos; i<cant+1; i++)
					texto[i-1] = texto[i];
				texto[cant] = '\0';
				cant--;
				setCursorPos(--pos);
			}
		break;
		case (SDLK_TAB):
			parent->NextFocus(this);
		break;
		case (SDLK_RSHIFT):
		case (SDLK_LSHIFT):
			shift = false;
		break;
		case (SDLK_LEFT):
			setCursorPos(--pos);
		break;
		case (SDLK_RIGHT):
			setCursorPos(++pos);
		break;
		case (SDLK_HOME):
			setCursorPos(0);
		break;
		case (SDLK_END):
			setCursorPos(cant);
		break;
		case (SDLK_DELETE):
			if (pos != cant) {
				for(int i = pos; i<cant+1; i++)
					texto[i] = texto[i+1];
				texto[cant] = '\0';
				cant--;
				setCursorPos(pos);
			}
		break;
		case (SDLK_INSERT):
			insert = !insert;
		break;
		case (SDLK_UP):
		case (SDLK_DOWN):
		case (SDLK_F1):
		case (SDLK_F2):
		case (SDLK_F3):
		case (SDLK_F4):
		case (SDLK_F5):
		case (SDLK_F6):
		case (SDLK_F7):
		case (SDLK_F8):
		case (SDLK_F9):
		case (SDLK_F10):
		case (SDLK_F11):
		case (SDLK_F12):
		case (SDLK_RETURN):
			break;
		default:
			if (cant != total) {
					if (insert)
						for(int i = cant; i != pos; i--)
							texto[i] = texto[i-1];

					if (shift)
						texto[pos] = toupper(key->keysym.sym);
					else
						texto[pos] = key->keysym.sym;
					cant++;
					setCursorPos(++pos);
					if (pos > cant) pos = cant;
					texto[cant] = '\0';
			}
	}
}

void TextField::eventMouseDown(SDL_MouseButtonEvent *e)
{
}

void TextField::eventMouseUp(SDL_MouseButtonEvent *e)
{
}

void TextField::eventMouseMove(SDL_MouseMotionEvent *e)
{
}

